﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace SyncInterProcessClient
{
    public partial class Form1 : Form
    {
        private SyncInterProcessWcfServiceReference.SyncMessagesServiceClient smsc_;
        private string log_file_;
        private int sync_id_;
        private Task responses_task_;
        private object lock_ = new object();

        public Form1()
        {
            InitializeComponent();

            // create log file
            log_file_ = Environment.GetFolderPath(Environment.SpecialFolder.MyDocuments) + $"\\SyncInterProcessClient.txt";
            // delete previous log
            File.WriteAllText(log_file_, "");
        }

        private void Shutdown()
        {
            if (smsc_ != null)
            {
                // close the service client
                smsc_.Close();
                // wait for responses task to end
                responses_task_.Wait();
                // dispose of the task
                responses_task_.Dispose();
                smsc_ = null;
            }
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            // create the service client
            smsc_ = new SyncInterProcessWcfServiceReference.SyncMessagesServiceClient();
            // start responses task
            responses_task_ = Task.Run(() =>
            {
                // responses task
                while (smsc_.State != System.ServiceModel.CommunicationState.Closed)
                {
                    int[] ids;
                    lock (lock_)
                        ids = new int[] { sync_id_ };

                    try
                    {
                        // get responses
                        string responses = smsc_.GetResponses(ids);
                        // synchronize IDE
                        basicIdeCtl1.Synchronize(responses, 0);
                    }
                    catch
                    {
                    }
                }
            });
        }

        private void Form1_Activated(object sender, EventArgs e)
        {
            if (!basicIdeCtl1.SynchronizedEdit)
            {
                // turn on synchronized editing
                basicIdeCtl1.SynchronizedEdit = true;
                // share the synchronized id with the get responses task
                lock (lock_)
                    sync_id_ = basicIdeCtl1.SynchronizedId;
            }
        }

        private void Form1_FormClosed(object sender, FormClosedEventArgs e)
        {
            Shutdown();
        }

        private void basicIdeCtl1_Disconnected(object sender, EventArgs e)
        {
            Shutdown();
        }

        private void basicIdeCtl1_Synchronizing(object sender, WinWrap.Basic.Classic.SynchronizingEventArgs e)
        {
            if (smsc_ != null)
            {
                string requests = e.Param;
                // log requests
                if (requests != null && requests != "[]")
                    File.AppendAllText(log_file_, $"-- Id={basicIdeCtl1.SynchronizedId} Synchronizing --\r\n{requests}\r\n\r\n");

                // send requests to the service
                smsc_.SendRequests(requests);
            }
        }
    }
}
