﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.Windows.Threading;

namespace SyncInterProcessClient
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        private SyncInterProcessWcfServiceReference.SyncMessagesServiceClient smsc_;
        private string log_file_;
        private bool attaching_;
        private int sync_id_;
        private Task responses_task_;
        private object lock_ = new object();

        public MainWindow()
        {
            InitializeComponent();

            // create log file
            log_file_ = Environment.GetFolderPath(Environment.SpecialFolder.MyDocuments) + $"\\SyncInterProcessClient.txt";
            // delete previous log
            File.WriteAllText(log_file_, "");
        }

        private void Shutdown()
        {
            if (smsc_ != null)
            {
                // close the service client
                smsc_.Close();
                // wait for responses task to end
                responses_task_.Wait();
                // dispose of the task
                responses_task_.Dispose();
                smsc_ = null;
            }
        }

        private void Window_Loaded(object sender, RoutedEventArgs e)
        {
            // create the service client
            smsc_ = new SyncInterProcessWcfServiceReference.SyncMessagesServiceClient();
            // start responses task
            responses_task_ = Task.Run(() =>
            {
                // responses task
                while (smsc_.State != System.ServiceModel.CommunicationState.Closed)
                {
                    int[] ids;
                    lock (lock_)
                        ids = new int[] { sync_id_ };

                    try
                    {
                        // get responses
                        string responses = smsc_.GetResponses(ids);
                        // synchronize IDE
                        basicIdeCtl1.Synchronize(responses, 0);
                    }
                    catch
                    {
                    }
                }
            });
        }

        private void Window_Activated(object sender, EventArgs e)
        {
            if (!attaching_)
            {
                attaching_ = true;
                // The responses task calls basicIdeCtl1.Synchronize
                // which using the Dispather queue to post the call
                // from the response task to the UI thread.
                // A dispatcher deadlock occurs if SynchronizedEdit is
                // set to true during the Window_Activated event.
                // Posting the dispatcher queue avoids the deadlock.
                Dispatcher.BeginInvoke((Action)(() =>
                {
                    // turn on synchronized editing
                    basicIdeCtl1.SynchronizedEdit = true;
                    // share the synchronized id with the get responses task
                    lock (lock_)
                        sync_id_ = basicIdeCtl1.SynchronizedId;
                }));
            }
        }

        private void Window_Closed(object sender, EventArgs e)
        {
            Shutdown();
        }

        private void basicIdeCtl1_Disconnected(object sender, EventArgs e)
        {
            Shutdown();
        }

        private void basicIdeCtl1_Synchronizing(object sender, WinWrap.Basic.Classic.SynchronizingEventArgs e)
        {
            if (smsc_ != null)
            {
                string requests = e.Param;
                // log requests
                if (requests != null && requests != "[]")
                    File.AppendAllText(log_file_, $"-- Id={basicIdeCtl1.SynchronizedId} Synchronizing --\r\n{requests}\r\n\r\n");

                // send requests to the service
                smsc_.SendRequests(requests);
            }
        }
    }
}
