﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Runtime.Serialization;
using System.ServiceModel;
using System.Text;
using System.Threading;

using WinWrap.Basic;
using WinWrap.Basic.Classic;

namespace SyncInterProcessWcfService
{
    // NOTE: In order to launch WCF Test Client for testing this service, please select WinWrapBasicService.svc or WinWrapBasicService.svc.cs at the Solution Explorer and start debugging.
    [ServiceBehavior(InstanceContextMode = InstanceContextMode.Single /*, ConcurrencyMode = ConcurrencyMode.Multiple*/)]
    public class SyncMessagesService : ISyncMessagesService
    {
        BasicThread basic_thread_;
        string log_file_;
        object lock_ = new object();
        WWB.SynchronizingQueues responses_sqs_ = new WWB.SynchronizingQueues();

        public SyncMessagesService()
        {
        }

        // GetResponses operation
        public string GetResponses(int[] ids)
        {
            string responses = null;
            WWB.SynchronizingQueue sq = new WWB.SynchronizingQueue(0);
            for (int i = 0; i < 20; ++i)
            {
                // get all pending responses
                lock (lock_)
                    foreach (int id in ids)
                        sq.Enqueue(responses_sqs_.Dequeue(id));

                responses = sq.DequeueAll();
                if (responses != null)
                    break;

                // wait for more responses
                Thread.Sleep(50);
            }

            return responses;
        }

        // SendRequests operation
        public void SendRequests(string requests)
        {
            if (basic_thread_ == null)
            {
                // authorizing application name is:
                // "@" + System.ServiceModel.OperationContext.Current.Host.Description.ServiceType.Assembly.FullName
                Util.SetCurrentServiceType();

                log_file_ = Environment.GetFolderPath(Environment.SpecialFolder.MyDocuments) + "\\SyncMessagesService.txt";
                File.WriteAllText(log_file_, "");
                basic_thread_ = new BasicThread();
                SynchronizationContext sc = new SynchronizationContext();
                basic_thread_.SendAction(basic =>
                {
                    // configure basic
                    basic.Synchronizing += (sender2, e2) =>
                    {
                        // response/notification from the remote BasicNoUIObj
                        lock (lock_)
                            responses_sqs_.Enqueue(e2.Param, e2.Id);
                        Log(e2.Param);
                    };
                    Util.IgnoreDialogs = true;
                    // set the BasicNoUIObj's secret
                    basic.Secret = new Guid("00000000-0000-0000-0000-000000000001");
                    // initialize basic in the remote thread
                    basic.Initialize();
                    // block unsafe scripting
                    basic.Sandboxed = true;
                    // block UI instructions
                    basic.BlockedKeywords = "AboutWinWrapBasic Beep Dialog GetFilePath InputBox MsgBox ShowPopupMenu";
                    // restrict file access
                    basic.VirtualFileSystem = new WWB.MyFileSystem(Environment.GetFolderPath(Environment.SpecialFolder.MyDocuments) + @"\SyncInterProcess");
                    // turn on NoUI's synchronized edit
                    basic.SynchronizedEdit = true;
                });
            }

            Log(requests);
            basic_thread_.PostAction(basic => basic.Synchronize(requests, 0));
        }

        private void Log(string param)
        {
            if (param != "[]")
                lock (lock_)
                    File.AppendAllText(log_file_, param + "\r\n\r\n");
        }
    }
}
